C***********************************
C***********************************
C***********************************
C       Make sure you set find_mwaits.inc to have the correct file format
C       monitor file you are looking at, the file format has changed over
C       the years.
C       If you are on a newer version of FORTRAN but using an old record
C       format then you MUST!!!! add in /ALIGNMENT=RECORD=PACKED or the
C       prgram will crash when run or will show 17-NOV-1858 as the start
C       date in the header.
C
C       FORTRAN /ALIGNMENT=(COMMONS=(PACKED),RECORDS=PACKED)/WARN=NOALIGN/LIST FIND_COMPUTES
C
C***********************************
C***********************************
C***********************************
C***********************************


C*
	PROGRAM FIND_COMPUTES
C*
C*	This program will read through a MONITOR/RECORD file and find any
C*	processes report how many COMpute processes there are. This
C*	can then be read into EXCEL for Graphing.
C*
C******************************************************************************
C*	The format for calling this program is
C*	COMPUTES input_file_name
C*		/OUTPUT=output_file_name 		(/OUTPUT=SYS$OUTPUT)
C*		/[NO]COUNT=number			(/COUNT=4)
C*
C*****************************************************************************

	INCLUDE '($STSDEF)'
	INCLUDE	'FIND_MWAITS.INC'

	CHARACTER*16384	RECORD_STRING  ! The Process record can be over 8500

	CHARACTER*80	FILE_NAME,
	1		OUTNAME,
	2		DATE_OUT
	CHARACTER*25	RECORD_TEXT
	CHARACTER*5	COMPUTE_COUNT

	INTEGER*4	DATE_LEN
	INTEGER*2	LENGTH,
	1		OUTNAME_SIZE
	LOGICAL		THERE

	INTEGER		CLI$PRESENT,
	1		CLI$GET_VALUE
	EXTERNAL	CLI$_PRESENT,
	1		CLI$_DEFAULTED
	CHARACTER	TAB
	DATA		TAB/"11"/
C*
C*            Get the input file
C*
	ISTATUS = CLI$GET_VALUE ('INFILE',FILE_NAME)
	IF (.NOT. ISTATUS) CALL LIB$STOP (%VAL(ISTATUS))

C*
C*		Check to see of the file is there.
C*
	INQUIRE(FILE=FILE_NAME,EXIST=THERE)
	IF (.NOT.THERE) THEN
		TYPE 100, FILE_NAME
		GOTO 9999
	ENDIF
100	FORMAT('0NO SUCH FILE: ',A)

	IF ((CLI$PRESENT ('OUTPUT') .EQ. %LOC(CLI$_PRESENT)) .OR.
	1	(CLI$PRESENT ('OUTPUT') .EQ. %LOC(CLI$_DEFAULTED))) THEN

		ISTATUS = CLI$GET_VALUE('OUTPUT',OUTNAME,OUTNAME_SIZE)
		IF (.NOT. ISTATUS) CALL LIB$STOP(%VAL(ISTATUS))
	ENDIF

	OPEN(UNIT=20,
	1	FILE=OUTNAME(1:OUTNAME_SIZE),
	2	STATUS='NEW',
	3	FORM='FORMATTED')


C*
C*	Check to see if the user wants to count the COMputable Processes.
C*	Otherwise set the minimum value to a high number that should
C*	never be exceeded.
C*
	IF ((CLI$PRESENT ('COUNT') .EQ. %LOC(CLI$_PRESENT)) .OR.
	1	(CLI$PRESENT ('COUNT') .EQ. %LOC(CLI$_DEFAULTED))) THEN
			ISTATUS = CLI$GET_VALUE('COUNT',COMPUTE_COUNT,LENGTH)
			IF (.NOT. ISTATUS) CALL LIB$STOP (%VAL(ISTATUS))
			DECODE (LENGTH,500,COMPUTE_COUNT(1:LENGTH))
	1			COMPUTE_COUNT_VALUE
	ELSE
		COMPUTE_COUNT_VALUE = 999
	ENDIF
500	FORMAT(I<LENGTH>)

C*
C*	Zero our count of Computable records for the first pass.
C*
	COMPUTABLE = 0

C*
C*	Open the Monitor File, use SHARED just in case you are reading an
C*	active recording file.
C*
	OPEN (UNIT=10, FILE=FILE_NAME,STATUS='OLD',READONLY,SHARED)


	WRITE (20,510) TAB,TAB,TAB
510	FORMAT(1X,'NODE',A1,'DATE',A1,'COMPUTABLE',A1,'TOTAL PROCESSES')

10	READ (10,110,END=1000) LENGTH, RECORD_STRING
110	FORMAT (Q,A)

C*
C*	Extract the Record Type Field.
C*
	CALL COPY_STRING_TO_RECORD (RECORD_STRING,CLASS_HEADER_REC.OVERLAY
	1		,0,LEN(CLASS_HEADER_REC.OVERLAY))

C*
C*	The Process Record and System Information Record are the only ones we
C*	are concerned with in this program.
C*
	IF (CLASS_HEADER_REC.MNR_CLS$B_TYPE .EQ. MNR_SYI$_TYPE)
	1	CALL SYSTEM_INFO_RECORD (CLASS_HEADER_REC,RECORD_STRING)

	IF (CLASS_HEADER_REC.MNR_CLS$B_TYPE .EQ. MNR_PRO$_TYPE)
	1	CALL PROCESS_RECORD (CLASS_HEADER_REC,RECORD_STRING)

	GOTO 10

1000	CLOSE (UNIT=10)
	CLOSE (unit=20)
9999	END

C******************************************************************************
C*
C*	This subroutine simply copies one string to another. It is used
C*	to copy parts of the string we read in to the proper record format
C*
	SUBROUTINE COPY_STRING_TO_RECORD
	1	(INSTRING,OUTSTRING,START_POS,NUM_CHARS)

	CHARACTER*(*)	INSTRING
	CHARACTER*(*)	OUTSTRING
	INTEGER		START_POS,
	1		NUM_CHARS

	DO I = 1, NUM_CHARS
		OUTSTRING(I:I) = INSTRING(START_POS+I:START_POS+I)
	END DO

	RETURN
	END

C******************************************************************************
C*
C*	This subroutine performs the processing that we want when we hit
C*	a System Information Record (The system information record
C*	should be the second record in the file in most cases.)
C*
	SUBROUTINE SYSTEM_INFO_RECORD (CLASS_HEADER_REC,RECORD_STRING)

	INCLUDE	'FIND_MWAITS.INC'

	CHARACTER*(*)	RECORD_STRING

	CALL COPY_STRING_TO_RECORD(RECORD_STRING,
	1		SYSTEM_INFO_REC.OVERLAY,0,
	2		LEN(SYSTEM_INFO_REC.OVERLAY))

	NODE_LENGTH = ICHAR(SYSTEM_INFO_REC.MNR_SYI$T_NODENAME(1:1))
	NODE = SYSTEM_INFO_REC.MNR_SYI$T_NODENAME(2:NODE_LENGTH+1)

	RETURN
	END

C******************************************************************************
C*
C*	This subroutine performs the processing that we want when we hit
C*	a PROCESS RECORD.
C*
	SUBROUTINE PROCESS_RECORD (CLASS_HEADER_REC,RECORD_STRING)

	INCLUDE '($LIBDTDEF)'
	INCLUDE	'FIND_MWAITS.INC'

	CHARACTER*(*)	RECORD_STRING
	CHARACTER*80	DATE_OUT
	INTEGER*4	DATE_LEN,
	1		COMPUTABLE
	CHARACTER	TAB
	DATA		TAB/"11"/

	CALL COPY_STRING_TO_RECORD(RECORD_STRING,
	1		CLASS_PREFIX_REC.OVERLAY,
	2		LEN(CLASS_HEADER_REC.OVERLAY),
	3		LEN(CLASS_PREFIX_REC.OVERLAY))

C*
C*	Get the date/time that the record was created
C*
	CALL LIB$FORMAT_DATE_TIME(DATE_OUT,
	1		CLASS_HEADER_REC.MNR_CLS$Q_STAMP,,TIME_LEN,)
C*	2		LIB$M_TIME_FIELDS)

C*
C*	Loop through each of the processes held in this record.
C*
	DO I = 1, CLASS_PREFIX_REC.MNR_CMP$L_ELTCT
		CALL COPY_STRING_TO_RECORD(RECORD_STRING,
	1		PROCESS_REC.OVERLAY,
	2		LEN(CLASS_PREFIX_REC.OVERLAY)+
	3		LEN(CLASS_HEADER_REC.OVERLAY)+((I-1)*
	4		LEN(PROCESS_REC.OVERLAY)),
	5		LEN(PROCESS_REC.OVERLAY))

C*
C*	Keep track of the number of Computable (or Computable-Outswapped)
C*	processes so we can report if there are too many.
C*
		IF ((PROCESS_REC.MNR_PRO$W_STATE .EQ. SCH$C_COM) .OR.
	1		(PROCESS_REC.MNR_PRO$W_STATE .EQ. SCH$C_COMO))
	2			COMPUTABLE = COMPUTABLE + 1

	END DO

C*
C*	Report if there are too many COM (including COMO) processes (unless
C*	we still have to read in another record to complete this snapshot).
C*
	IF (.NOT. (CLASS_HEADER_REC.MNR_CLS$B_FLAGS .AND. 1)) THEN

		IF (COMPUTABLE .GE. COMPUTE_COUNT_VALUE) THEN
			WRITE (20,250) NODE,TAB,DATE_OUT,TAB,
	1			COMPUTABLE,TAB,
	2			CLASS_PREFIX_REC.MNR_CMP$L_ELTCT
		ENDIF

		COMPUTABLE = 0
	ENDIF

250	FORMAT(1X,A<NODE_LENGTH>,A1,A<TIME_LEN>,A1,I3,A1,I5)

	RETURN
	END

